local AceGUI = LibStub("AceGUI-3.0");
local Spreadsheet = LibStub("LibSpreadsheet-1.0");


if ( GnomexcelDB == nil ) then
	GnomexcelDB = {
		books = {}
	};
end

Gnomexcel = {
	mainWin = nil;
	toolBox = nil;
	fileTree = nil;
	revision = tonumber(("$Rev: 06$"):match("(%d+)")) or "0 (dev copy)";
};

function Gnomexcel:AddRow(num)
	local start = Gnomexcel.book.selectedSheet.selectStartRow or false;
	if ( start ) then
		for i = 1, num do
			Gnomexcel.book.selectedSheet:AddRowAt(start, "","","","","","","","","","","","","","","","","","","","","","","","","","");
		end
	else
		for i = 1, num do
			Gnomexcel.book.selectedSheet:AddRow("","","","","","","","","","","","","","","","","","","","","","","","","","");
		end
	end
	Gnomexcel.book:Render_Update();
end

function Gnomexcel:AddColumn(num)
	local start = Gnomexcel.book.selectedSheet.selectStartCol or false;
	if ( start ) then
		for i = 1, num do
			Gnomexcel.book.selectedSheet:AddColumnAt(start,
		{
			type="string",
			name = i,
			background={0.14,0.14,0.14},
			color={1,0.95,0.4}
		}
		);
		end
	else
		for i = 1, num do
			Gnomexcel.book.selectedSheet:AddColumn(
		{
			type="string",
			name = i,
			background={0.14,0.14,0.14},
			color={1,0.95,0.4}
		}
		);
		end
	end
	Gnomexcel.book:Render_Update();
end

function Gnomexcel:SetStat(prop, val)
	if ( Gnomexcel.book ) then
		if ( val ~= "void" ) then
			for k, cell in pairs(Gnomexcel.book.selectedSheet:GetSelectedCells()) do
				cell[prop] = val;
				cell.u = true;
			end
		end
		Gnomexcel.book:Render_Real();
	end
end

function Gnomexcel:Wipe()
	for k, cell in pairs(Gnomexcel.book.selectedSheet:GetSelectedCells()) do
		cell.text = "";
		cell.data = nil;
		cell.MathData = nil;
		cell.MathFuntion = nil;
	end
	Gnomexcel.book:Render_Update();
end

function Gnomexcel:DeleteRows()
	for k, row in pairs(Gnomexcel.book.selectedSheet:GetSelectedRows()) do
		Gnomexcel.book.selectedSheet:DeleteRow(row);
	end
	Gnomexcel.book:Render_Update();
end

function Gnomexcel:DeleteCols()
	local x = 0;
	for k, col in pairs(Gnomexcel.book.selectedSheet:GetSelectedCols()) do
		Gnomexcel.book.selectedSheet:DeleteColumn(col+x);
		x = x - 1;
	end
	Gnomexcel.book:Render_Update();
end


function Gnomexcel:CreateBook(title)
	local book = Spreadsheet:Book(title);
	book:StrictMode(true);
	local sheet = book:Add("Sheet 1");
	sheet.background = {1,1,1,0.4};

	for i = 1, 15 do
		sheet:AddColumn(
		{
			type="string",
			name = i,
			background={0.14,0.14,0.14},
			color={1,0.95,0.4}
		}
		);
	end
	for i = 1, 15 do
		sheet:AddRow("","","","","","","","","","","","","","","","","","","","","","","","","","");
	end
	sheet.isInteractive = true;
	book:EnableMaths(true);
	local hash = string.format("%8x-%8x-%8x", math.random(time()), math.random(time()/3), math.random(time()-400));
	book.hash = hash;

	table.insert(GnomexcelDB.books,
		{
		author = GetUnitName("Player"),
		realm = GetRealmName(),
		created = time(),
		file = book,
		hash = hash;
		}
	);
end

function Gnomexcel:DeleteBook()
	if ( Gnomexcel.book ~= nil ) then
		Gnomexcel.book:Hide();
		Gnomexcel.book:Close();
	end
	for key,book in pairs(GnomexcelDB.books) do
		if ( book.hash == Gnomexcel.book.hash ) then
			table.remove(GnomexcelDB.books, key);
			break;
		end
	end
	Gnomexcel:GetBooks();
end

function Gnomexcel:GetBooks()
	local tbl = {
		{
			value = "$$Function=NewBook$$",
			text = "Create a new book",
			icon = "Interface\\Icons\\INV_Misc_Book_06",
		},
		{
			value = "$$Empty$$",
			text = "   ",
			icon = "",
			disabled = true,
		},
		{
			value = "$$Empty$$",
			text = "|cffffffffBooks in your library:|r",
			icon = "",
			disabled = true,
		},

	};
	for key, book in pairs(GnomexcelDB.books) do
		table.insert(tbl, {
			value = "$$Book=" .. key .. "$$",
			text = book.file.title,
			icon = ("Interface\\Icons\\INV_Misc_Book_%02u"):format(math.random(0, 4) + 7);
		}
		);
	end
	Gnomexcel.fileTree:SetTree(tbl);
	Gnomexcel.fileTree:SetUserData("tree", tbl);

	Gnomexcel.fileTree:SetCallback("OnClick",
		function(caller, event, value)
            local subtree, subval = (value):match("%$(%a+)=([^%$]+)%$");
			if ( subtree == "Function" ) then
				if ( subval == "NewBook" ) then
					local frame = AceGUI:Create("Frame");
					frame:SetWidth(300);
					frame:SetHeight(100);
					frame:SetTitle("Create new book");
					local box = AceGUI:Create("EditBox");
					box:SetText("Enter the name of the book");
					frame:AddChild(box);
					frame:Show();
					box:SetCallback("OnEnterPressed",
						function(caller, event, title)
							Gnomexcel:CreateBook(title);
							Gnomexcel:GetBooks();
							frame:ReleaseChildren();
							frame:Release();
						end
					);
				end
			end
			if ( subtree == "Book" ) then
				if ( Gnomexcel.book ~= nil ) then
					Gnomexcel.book:Hide();
					Gnomexcel.book:Close();
				end
				Gnomexcel.book = Spreadsheet:Load(GnomexcelDB.books[tonumber(subval)].file);
				Gnomexcel.book:SetStyle("sheet");
				Gnomexcel.book:ShowSelector(true);
				Gnomexcel.book:Select();
				frame = Gnomexcel.book:Render();
				Gnomexcel.container:ReleaseChildren();
				frame:SetParent(Gnomexcel.container.frame);
				frame:SetPoint("TOPLEFT", Gnomexcel.container.frame, "TOPLEFT", 10, -10);
				frame:SetPoint("BOTTOMRIGHT", Gnomexcel.container.frame, "BOTTOMRIGHT", -10, 10);
				frame:Show();
			end
		end
	);
	Gnomexcel.fileTree:SetCallback("OnButtonEnter",
		function(caller, event, value)
			local subtree, subval = (value):match("%$(%a+)=([^%$]+)%$");
			if ( subtree == "Book" ) then
				if ( tonumber(subval) ~= 0 ) then
					local book = GnomexcelDB.books[tonumber(subval)];
					GameTooltip:SetOwner(caller.frame, "ANCHOR_TOPLEFT");
					GameTooltip:ClearLines();
					GameTooltip:SetText(book.file.title .. ":");
					GameTooltip:AddDoubleLine("|cff55DD11Author:|r", book.author .. " of " .. book.realm);
					GameTooltip:AddDoubleLine("|cff55DD11Created:|r", date(nil, book.created));
					GameTooltip:Show();
				end
			end
		end);
	Gnomexcel.fileTree:SetCallback("OnButtonLeave", function() GameTooltip:Hide(); end);
	Gnomexcel.fileTree:EnableButtonTooltips(false);
end

function Gnomexcel:MakeToolbox()

	local group = AceGUI:Create("SimpleGroup");
	group:SetWidth(450);
	group:SetLayout("Flow");

	local icon = AceGUI:Create("Icon");
	icon:SetWidth(48);
	icon:SetImage([[Interface\Icons\INV_Misc_Bomb_07]]);
	icon:SetImageSize(32,32);
	icon:SetCallback("OnEnter", function(caller) GameTooltip:ClearLines(); GameTooltip:SetOwner(caller.frame, "ANCHOR_BOTTOM"); GameTooltip:SetText("Delete book"); GameTooltip:Show(); end);
	icon:SetCallback("OnLeave", function() GameTooltip:Hide(); end);
	icon:SetCallback("OnClick", function() if ( Gnomexcel.book ) then Gnomexcel:DeleteBook() end end );
	group:AddChild(icon);

	icon = AceGUI:Create("Icon");

	icon:SetImageSize(32,32);
	icon:SetWidth(48);
	icon:SetCallback("OnEnter", function(caller) GameTooltip:ClearLines(); GameTooltip:SetOwner(caller.frame, "ANCHOR_BOTTOM"); GameTooltip:SetText("Add sheet"); GameTooltip:Show(); end);
	icon:SetCallback("OnLeave", function() GameTooltip:Hide(); end);
	icon:SetImage([[Interface\Icons\INV_Misc_Bandage_10]]);
	icon:SetCallback("OnClick", function()
	if ( Gnomexcel.book ) then
		local sheet = Gnomexcel.book:Add(nil);
		sheet.background = {1,1,1,0.4};

	for i = 1, 15 do
		sheet:AddColumn(
		{
			type="string",
			name = i,
			background={0.14,0.14,0.14},
			color={1,0.95,0.4}
		}
		);
	end
	for i = 1, 15 do
		sheet:AddRow("","","","","","","","","","","","","","","","","","","","","","","","","","");
	end
	sheet.isInteractive = true;
		Gnomexcel.book:Render(); end end);
	group:AddChild(icon);




	icon = AceGUI:Create("Icon");
	icon:SetImageSize(32,32);
	icon:SetWidth(48);
	icon:SetCallback("OnEnter", function(caller) GameTooltip:ClearLines(); GameTooltip:SetOwner(caller.frame, "ANCHOR_BOTTOM"); GameTooltip:SetText("Delete sheet"); GameTooltip:Show(); end);
	icon:SetCallback("OnLeave", function() GameTooltip:Hide(); end);
	icon:SetImage([[Interface\Icons\INV_Misc_Map_01]]);
	icon:SetCallback("OnClick", function() if ( Gnomexcel.book ) then Gnomexcel.book:Delete(nil); Gnomexcel.book:Render(); end end);
	group:AddChild(icon);



	local menu = {
    { text = "Add rows/columns:", isTitle = true},
    { text = "Add a single column", func = function() Gnomexcel:AddColumn(1) end },
    { text = "Add 5 columns", func = function() Gnomexcel:AddColumn(5) end },
	{ text = "Add 10 columns", func = function() Gnomexcel:AddColumn(10) end },
    { text = " "},
	{ text = "Add a single row", func = function() Gnomexcel:AddRow(1) end },
    { text = "Add 5 rows", func = function() Gnomexcel:AddRow(5) end },
	{ text = "Add 10 rows", func = function() Gnomexcel:AddRow(10) end },
	{ text = "Add 20 rows", func = function() Gnomexcel:AddRow(20) end },
	{ text = "Add 50 rows", func = function() Gnomexcel:AddRow(50) end },
	{ text = "",},
	{ text ="Close menu" }
    };

	icon = AceGUI:Create("Icon");
	icon:SetImage([[Interface\Icons\Spell_ChargePositive]]);
	icon:SetImageSize(32,32);
	icon:SetWidth(48);
	icon:SetCallback("OnEnter", function(caller) GameTooltip:ClearLines(); GameTooltip:SetOwner(caller.frame, "ANCHOR_BOTTOM"); GameTooltip:SetText("Add rows/columns"); GameTooltip:AddLine("|cff55EE22Tip: If you have a row, column or cell selected, any added |nrows/columns will be inserted BEFORE that selection.|r"); GameTooltip:Show(); end);
	icon:SetCallback("OnLeave", function() GameTooltip:Hide(); end);
	icon:SetCallback("OnClick", function() EasyMenu(menu, Gnomexcel.menuFrame, "cursor", 0 , 0, "MENU"); end);

	group:AddChild(icon);


	local menu = {
    { text = "Delete rows/columns:", isTitle = true},
    { text = "Delete row(s)", func = function() Gnomexcel:DeleteRows() end },
    { text = "Delete column(s)", func = function() Gnomexcel:DeleteCols() end },
	{ text = "",},
	{ text ="Close menu" }
    };
	icon = AceGUI:Create("Icon");
	icon:SetImageSize(32,32);
	icon:SetWidth(48);
	icon:SetCallback("OnEnter", function(caller) GameTooltip:ClearLines(); GameTooltip:SetOwner(caller.frame, "ANCHOR_BOTTOM"); GameTooltip:SetText("Delete rows/columns"); GameTooltip:AddLine("|cff55EE22Tip: Any rows or columns following the selected|n ones will be shifted up or to the left upon deleting.|nIf you just wish to clear the cells,|nselect them and use the [eraser] button.|r");GameTooltip:Show(); end);
	icon:SetCallback("OnLeave", function() GameTooltip:Hide(); end);
	icon:SetImage([[Interface\Icons\Spell_ChargeNegative]]);
	icon:SetCallback("OnClick", function() EasyMenu(menu, Gnomexcel.menuFrame, "cursor", 0 , 0, "MENU"); end);
	group:AddChild(icon);


	icon = AceGUI:Create("Icon");
	icon:SetImageSize(32,32);
	icon:SetWidth(48);
	icon:SetCallback("OnEnter", function(caller) GameTooltip:ClearLines(); GameTooltip:SetOwner(caller.frame, "ANCHOR_BOTTOM"); GameTooltip:SetText("Send book"); GameTooltip:Show(); end);
	icon:SetCallback("OnLeave", function() GameTooltip:Hide(); end);
	icon:SetImage([[Interface\Icons\Inv_Letter_15]]);
	--icon:SetCallback("OnClick", function() if ( Gnomexcel.book ) then Gnomexcel.book.selectedSheet:icon(""); Gnomexcel.book:Render_ScrollFunction(); end end);
	group:AddChild(icon);

	icon = AceGUI:Create("Icon");
	icon:SetImageSize(32,32);
	icon:SetWidth(48);
	icon:SetCallback("OnEnter", function(caller) GameTooltip:ClearLines(); GameTooltip:SetOwner(caller.frame, "ANCHOR_BOTTOM"); GameTooltip:SetText("Export data"); GameTooltip:Show(); end);
	icon:SetCallback("OnLeave", function() GameTooltip:Hide(); end);
	icon:SetImage([[Interface\Icons\INV_Misc_PunchCards_Yellow]]);
	--icon:SetCallback("OnClick", function() if ( Gnomexcel.book ) then Gnomexcel.book.selectedSheet:icon(""); Gnomexcel.book:Render_ScrollFunction(); end end);
	group:AddChild(icon);






	icon = AceGUI:Create("Icon");
	icon:SetImageSize(32,32);
	icon:SetWidth(48);
	icon:SetCallback("OnEnter", function(caller) GameTooltip:ClearLines(); GameTooltip:SetOwner(caller.frame, "ANCHOR_BOTTOM"); GameTooltip:SetText("Settings"); GameTooltip:Show(); end);
	icon:SetCallback("OnLeave", function() GameTooltip:Hide(); end);
	icon:SetImage([[Interface\Icons\INV_Misc_EngGizmos_30]]);
	--icon:SetCallback("OnClick", function() EasyMenu(menu, Gnomexcel.menuFrame, "cursor", 0 , 0, "MENU"); end);
	group:AddChild(icon);

	Gnomexcel.toolBox:AddChild(group);



	group = AceGUI:Create("SimpleGroup");
	group:SetWidth(200);
	group:SetLayout("Flow");

	local menu = {
    { text = "Data type:", isTitle = true},
    { text = "Numbers", func = function() Gnomexcel:SetStat("t", "number"); end },
    { text = "Text", func = function()  Gnomexcel:SetStat("t", "string"); end },
    { text = "WoW Currency", func = function()  Gnomexcel:SetStat("t", "currency"); end },
	{ text = "WoW Item/Spell", func = function()  Gnomexcel:SetStat("t", "link"); end },
	{ text = "Booleans", func = function()  Gnomexcel:SetStat("t", "boolean"); end },
    };
	--[[
	INV_Misc_Spyglass_02



http://wowprogramming.com/BlizzArt/Icons/INV_Holiday_Christmas_Wrapping_01.png
http://wowprogramming.com/BlizzArt/Icons/INV_Misc_QuestionMark.png
]]


	icon = AceGUI:Create("Icon");
	icon:SetImageSize(32,32);
	icon:SetWidth(48);
	icon:SetCallback("OnEnter", function(caller) GameTooltip:ClearLines(); GameTooltip:SetOwner(caller.frame, "ANCHOR_BOTTOM"); GameTooltip:SetText("Data type"); GameTooltip:Show(); end);
	icon:SetCallback("OnLeave", function() GameTooltip:Hide(); end);
	icon:SetImage([[Interface\Icons\INV_Misc_QuestionMark]]);
	icon:SetCallback("OnClick", function() EasyMenu(menu, Gnomexcel.menuFrame, "cursor", 0 , 0, "MENU"); end);
	group:AddChild(icon);



	local menu = {
    { text = "Data precision:", isTitle = true},
    { text = "Whole numbers (12345)", func = function() Gnomexcel:SetStat("d", 0); end },
	{ text = "1 decimal  (1234.5)", func = function() Gnomexcel:SetStat("d", 1); end },
    { text = "2 decimals (123.45)", func = function()  Gnomexcel:SetStat("d", 2); end },
    { text = "3 decimals (12.345)", func = function()  Gnomexcel:SetStat("d", 3); end },
	{ text = "4 decimals (1.2345)", func = function()  Gnomexcel:SetStat("d", 4); end },
    };
	icon = AceGUI:Create("Icon");
	icon:SetImageSize(32,32);
	icon:SetWidth(48);
	icon:SetCallback("OnEnter", function(caller) GameTooltip:ClearLines(); GameTooltip:SetOwner(caller.frame, "ANCHOR_BOTTOM"); GameTooltip:SetText("Data precision"); GameTooltip:Show(); end);
	icon:SetCallback("OnLeave", function() GameTooltip:Hide(); end);
	icon:SetImage([[Interface\Icons\INV_Misc_Spyglass_02]]);
	icon:SetCallback("OnClick", function() EasyMenu(menu, Gnomexcel.menuFrame, "cursor", 0 , 0, "MENU"); end);
	group:AddChild(icon);


	local menu = {
    { text = "Prefix/Suffix:", isTitle = true},
	{ text = "(None)", func = function() Gnomexcel:SetStat("s", ""); Gnomexcel:SetStat("p", ""); end },
    { text = "% (Percent)", func = function() Gnomexcel:SetStat("s", "%");Gnomexcel:SetStat("p", ""); end },
	{ text = "mp5 (Mana per 5 seconds)", func = function()  Gnomexcel:SetStat("p", ""); Gnomexcel:SetStat("s", " mp5"); end },
	{ text = "dps (Damage per second)", func = function()  Gnomexcel:SetStat("p", ""); Gnomexcel:SetStat("s", " dps"); end },
	{ text = "hps (Healing per second)", func = function()  Gnomexcel:SetStat("p", ""); Gnomexcel:SetStat("s", " hps"); end },
	{ text = "tps (Threat per second)", func = function()  Gnomexcel:SetStat("p", ""); Gnomexcel:SetStat("s", " tps"); end },
    };
	icon = AceGUI:Create("Icon");
	icon:SetImageSize(32,32);
	icon:SetWidth(48);
	icon:SetCallback("OnEnter", function(caller) GameTooltip:ClearLines(); GameTooltip:SetOwner(caller.frame, "ANCHOR_BOTTOM"); GameTooltip:SetText("Suffix/Prefix"); GameTooltip:Show(); end);
	icon:SetCallback("OnLeave", function() GameTooltip:Hide(); end);
	icon:SetImage([[Interface\Icons\INV_Misc_Ribbon_01]]);
	icon:SetCallback("OnClick", function() EasyMenu(menu, Gnomexcel.menuFrame, "cursor", 0 , 0, "MENU"); end);
	group:AddChild(icon);

	icon = AceGUI:Create("Icon");
	icon:SetImageSize(32,32);
	icon:SetWidth(48);
	icon:SetCallback("OnEnter", function(caller) GameTooltip:ClearLines(); GameTooltip:SetOwner(caller.frame, "ANCHOR_BOTTOM"); GameTooltip:SetText("Erase selected cells"); GameTooltip:Show(); end);
	icon:SetCallback("OnLeave", function() GameTooltip:Hide(); end);
	icon:SetImage([[Interface\Icons\INV_Inscription_InkWhite01]]);
	icon:SetCallback("OnClick", function() Gnomexcel:Wipe(); end);
	group:AddChild(icon);


	local textColor = AceGUI:Create("ColorPicker");
	textColor:SetLabel("Text");
	textColor:SetWidth(60);
	textColor:SetColor(1,0.95,0.4);
	textColor:SetCallback("OnValueConfirmed",
		function(caller, event, r, g, b)
			if ( Gnomexcel.book ) then
				for k, cell in pairs(Gnomexcel.book.selectedSheet:GetSelectedCells()) do
					cell.c = {r,g,b,1};
					cell.u = true;
				end
				Gnomexcel.book:Render_Real();
			end
		end
	);
	group:AddChild(textColor);

	local background = AceGUI:Create("ColorPicker");
	background:SetLabel("Background");
	background:SetColor(0.14,0.14,0.14);
	background:SetWidth(89);
	background:SetCallback("OnValueConfirmed",
		function(caller, event, r, g, b)
			if ( Gnomexcel.book ) then
				for k, cell in pairs(Gnomexcel.book.selectedSheet:GetSelectedCells()) do
					cell.b = {r,g,b,1};
					cell.u = true;
				end
				Gnomexcel.book:Render_Real();
			end
		end
	);
	group:AddChild(background);

	Gnomexcel.toolBox:AddChild(group);
end


function Gnomexcel:MainWindow()
	local f = AceGUI:Create("Frame")
	f:SetCallback("OnClose",function(widget) f:Hide(); end)
	f:SetTitle("Gnomexcel rev. " .. Gnomexcel.revision);
	f:SetStatusText("Welcome!");
	f:SetLayout("Flow")
	f:SetWidth(730);
	f:SetAutoAdjustHeight(false);
	f:SetHeight(480);

	local toolBox = AceGUI:Create("SimpleGroup");
	toolBox.width= "fill";
	toolBox:SetHeight(80);
	toolBox:SetLayout("Flow")
	toolBox:SetAutoAdjustHeight(false);
	Gnomexcel.toolBox = toolBox;
	f:AddChild(toolBox);

	local fileTree = AceGUI:Create("TreeGroup");
	fileTree.width = "fill";
	fileTree.height = "fill";

	f:AddChild(fileTree);

	local container = AceGUI:Create("SimpleGroup");

	fileTree:SetLayout("fill");
	fileTree:AddChild(container);

	f.frame:SetScript("OnShow", function() Gnomexcel:GetBooks(); end);
	Gnomexcel.mainWin = f;
	Gnomexcel.container = container;
	f:Hide();
	Gnomexcel.book = book;
	Gnomexcel.fileTree = fileTree;
	Gnomexcel:MakeToolbox();

	local t = AceGUI:Create("Label");
	t:SetText("Welcome to Gnomexcel! To get started, select one of your previous books or create a new one to start making spreadsheets!");
	t:SetWidth(400);
	container:AddChild(t);

	Gnomexcel.menuFrame = CreateFrame("Frame", "GnomexcelMenu", UIParent, "UIDropDownMenuTemplate")
end
do
	Gnomexcel:MainWindow();
	SLASH_GNOMEXCEL1, SLASH_GNOMEXCEL2 = '/gxl', '/gnomexcel';
function SlashCmdList.GNOMEXCEL(msg, editbox)
 Gnomexcel.mainWin:Show();
end

end
